// Copyright (c) 2017, Muienog AB. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library data_table_component;
import 'dart:collection' show SplayTreeMap;
import 'package:angular2/core.dart';
import 'package:fo_components/components/icon_component/icon_component.dart';
import 'package:fo_components/pipes/map_to_keys_pipe.dart';
import 'package:fo_components/pipes/map_to_values_pipe.dart';
import 'package:fo_components/pipes/uppercase_pipe.dart';

@Component(
    selector: 'fo-data-table',
    styleUrls: const ['data_table_component.css'],
    templateUrl: 'data_table_component.html',
    directives: const [IconComponent],
    preserveWhitespace: false,
    pipes: const [MapToKeysPipe, MapToValuesPipe, UppercasePipe])

class DataTableComponent extends ComponentState
{
  DataTableComponent();

  void onCellClick(String data)
  {
    foCellClick.emit(data);
  }

  void onRowClick(String id)
  {
    _selectedId = id;
    foRowClick.emit(id);
  }

  void step(int steps)
  {
    int cIndex = currentIndices.first + (_rows * steps);
    int startIndex = cIndex;
    List<int> buffer = new List();

    while (cIndex >= 0 && cIndex < startIndex + _rows && cIndex < _data.length)
    {
      buffer.add(cIndex);
      cIndex += 1;
    }
    if (buffer.isNotEmpty) currentIndices = new List.from(buffer, growable: false);
  }

  void toggleSort(String column)
  {
    sortColumn = column;
    sortOrder = (sortOrder == "ASC") ? "DESC" : "ASC";

    if (sortColumn == "ID") data = new Map.from(new SplayTreeMap.from(_data, compare));
    else
    {
      data = new Map.from(new SplayTreeMap.from(_data, (key1, key2) => compareColumn(key1, key2, sortColumn)));
    }
  }

  int compare(String key1, String key2)
  {
    num num1, num2;
    try
    {
      num1 = num.parse(key1);
      num2 = num.parse(key2);
    } on FormatException
    {
      // NaN, compare strings
      if (sortOrder == "ASC") return key1.compareTo(key2);
      else return key2.compareTo(key1);
    }
    return (sortOrder == "ASC") ? num1 - num2 : num2 - num1;
  }

  int compareColumn(String key1, String key2, String column)
  {
    int value = compare(_data[key1][column], _data[key2][column]);
    if (value == 0) value = 1; // columns can have multiple values with the same value
    return value;
  }

  @Input()
  void set data(Map<String, Map<String, String>> value)
  {
    setState(() => _data = value);
  }

  @Input()
  void set rows(int value)
  {
    setState(()
    {
      _rows = value;
      currentIndices = new List(_rows);
      for (int i = 0; i < _rows; i++)
      {
        currentIndices[i] = i;
      }
    });
  }

  @Output('cellclick')
  final EventEmitter<String> foCellClick = new EventEmitter();

  @Output('rowclick')
  final EventEmitter<String> foRowClick = new EventEmitter();

  String get selectedId => _selectedId;
  String _selectedId;
  Map<String, Map<String, String>> get data => _data;
  Map<String, Map<String, String>> _data;

  int _rows = 5;
  List<int> currentIndices = [0, 1, 2, 3, 4];
  String sortColumn = "ID";
  String sortOrder = "DESC";
}
